/*****************************************************
This program was produced by the
CodeWizardAVR V2.05.0 Professional
Automatic Program Generator
 Copyright 1998-2010 Pavel Haiduc, HP InfoTech s.r.l.
http://www.hpinfotech.com

Project : LCmeter-2
Version : 1.0
Date    : 17.10.2018
Author  : RA4NAL
Company : Home
Comments: 

Chip type               : ATmega328P
Program type            : Application
AVR Core Clock frequency: 16,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 512

Connections to ArduinoProMini

Line    |    Port ATmega328    | Pin
--------|----------------------|----
ADC     |   ADC1 - PC1 IN AN   | A1
FRQ     |   OC1B - PB2 OUT 0   | D10
L/C     |          PD2 IN P D  | D2
0       |   INT1 - PD3 IN P D  | D3
        |                      |
DB4     |          PD4 OUT 0   | D4
DB5     |          PD5 OUT 0   | D5
DB6     |          PD6 OUT 0   | D6
DB7     |          PD7 OUT 0   | D7
RS      |          PB0 OUT 0   | D8
E       |          PB1 OUT 0   | D9
------------------------------------
LED     |          PB5 OUT 0   |
Not Used|       Others IN P D  |
***********************************************/

#include <mega328p.h>
#include <delay.h>
#include <stdlib.h>
#include <string.h>

#define NMAX  8        // max diap
#define maxCAL 0xC0    // max Calib value
#define defCAL 0x0A    // default Calib value
#define Umax 0x0380    // U max for band
#define Umin 0x00C0    // U min for band

#define pin_LC PIND.2  // L/C
#define pin_0  PIND.3  // Button "Set 0" (INT1)

// ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
  {
// reti
  }

#include "lcm_asm.c"    // asm
#include "lcd.c"        // LCD & EEPROM

void main(void)
{
// Declare your local variables here
unsigned char Ndiap = NMAX;
unsigned char Counter1 = 0;
unsigned char CalL[NMAX+1], CalC[NMAX+1];
unsigned char Flag, TMP;
unsigned int AdcRes;
unsigned long int Result;
unsigned char strResult[9] = {0,0,0,0,0,0,0,0,0};  // string for long int
unsigned char pattern[8]={0x00,0x11,0x11,0x11,0x13,0x1D,0x10,0x10};  // "mikro"

#include "initsfr_328.c"       // initialization SFR mega328
#asm ("sei")

// Read "0" value
for (TMP=0; TMP <= NMAX; TMP++ )
   {
     CalC[TMP] = EEPROM_read(TMP+1);         // "0" value for C
     CalL[TMP] = EEPROM_read(TMP+NMAX+2);    // "0" value for L
     if (CalC[TMP] >= maxCAL)
       {
         CalC[TMP] = defCAL;
         EEPROM_write((TMP+1), defCAL);
       }
     if (CalL[TMP] >= maxCAL)
       {
         CalL[TMP] = defCAL;
         EEPROM_write(TMP+NMAX+2, defCAL);
       }
   }

lcd_init();
LCD_build(pattern);            // create simbol "mikro"

while (1)
  {
    if (Ndiap > NMAX)
       Ndiap = NMAX;
    if (EIFR.INTF1)            // Push "Set 0"
// Calibration C or L
      {
        TMP = 10;
        while (TMP)            // Wait for relese button
          {
            delay_ms(2);
            if (pin_0 == 0) TMP = 10;
            else --TMP;
          }
        lcd_clear();
        lcd_goto(0,0);
        if (pin_LC == 0)
           lcd_puts("Calib C - open");
        else  lcd_puts("Calib L - short");
        lcd_goto(0,1);
        lcd_puts("Wait...");
        Flag=1;
        for (Counter1 =0; Counter1 <= NMAX; Counter1++)
          {
            AdcRes = RchF(Counter1,NMAX);            // Calibration
            if (AdcRes < maxCAL)
              {
                if (pin_LC == 0)                     // Mode C
                  {
                    CalC[Counter1] = (char)AdcRes;
                    TMP = EEPROM_read(Counter1+1);
                    if (CalC[Counter1] != TMP)
                       EEPROM_write((Counter1+1), CalC[Counter1]);
                  }
                else                                 // Mode L
                  {
                    CalL[Counter1] = (char)AdcRes;
                    TMP = EEPROM_read(Counter1+NMAX+2);
                    if (CalL[Counter1] != TMP)
                       EEPROM_write(Counter1+NMAX+2, CalL[Counter1]);
                  }
              }
            else
              {
                Flag=0;                              // Error
                if (pin_LC == 0)
                  CalC[Counter1] = defCAL;
                else
                  CalL[Counter1] = defCAL;
              }
          }
        lcd_goto(0,1);
        if (Flag)
          lcd_puts("Done   ");
        else
          lcd_puts("Error ");

        delay_ms(1000);
        lcd_clear();
        EIFR = 0x02;           // Clear flag INTF1
      }

// Measuring C or L
    AdcRes = RchF(Ndiap,NMAX);
    Flag = 1;
    if ((AdcRes > Umax) && (Ndiap < NMAX))
      {
        lcd_frq(Ndiap,AdcRes);         // Freq and lim to display
        ++Ndiap;
        Counter1 = 0;
        Flag = 0;
      }
    if ((AdcRes < Umin) && Ndiap)
      {
        lcd_frq(Ndiap,AdcRes);         // Freq and lim to display
        --Ndiap;
        Counter1 = 0;
        Flag = 0;
      }
    if (Flag)                          // L or C to display
      {
        if (Counter1 < 8)
         {
           lcd_frq(Ndiap,AdcRes);
           Counter1++;
         }  
        else 
           {
              lcd_spc(1);                 //Clear 1 line
           }          
        Result = AdcRes;

// +/- Calib value
        Flag = 0;                      // Sign "+"
        if (pin_LC == 0)               // Mode "C"
          {
            if (Result >= CalC[Ndiap]) Result = Result - CalC[Ndiap];
            else
              {
                Result = CalC[Ndiap] - Result;
                Flag = 1;              // Sign "-"
              }
          }
        else                           // Mode "L"
          {
            if (Result >= CalL[Ndiap]) Result = Result - CalL[Ndiap];
            else
              {
                Result = CalL[Ndiap] - Result;
                Flag = 1;              // Sign "-"
              }
          }

        TMP = Ndiap;
        while (TMP)                    // Count L or C
          {
            --TMP;
            Result <<=2;               // *4
          }

        ltoa(Result, strResult);  // long int to string
        TMP = prpeLC(strResult);  // Leave 3 significant digits, TMP=length of the string
        outIND(strResult, TMP, Flag);   // Output string to indicator
      }
  }
}

